﻿init python:
    from dataclasses import dataclass

    @dataclass
    class spell:
        name: str
        info: str
        icon: str

        health: int = 0
        mana: int = 0
        stamina: int = 0

        health_cost: int = 0
        mana_cost: int = 0
        stamina_cost: int = 0

        effect: str = ""
        targets: str = ""
        sound: str = ""

    @dataclass
    class fighter:
        name: str
        info: str
        image: str

        health: int
        mana: int
        stamina: int

        spells: list

        x: int = 0
        y: int = 0
        z: int = 1

        stat: str = "idle"

        def __post_init__(self):
            self.max_health: int = self.health
            self.max_mana: int = self.mana
            self.max_stamina: int = self.stamina


    @dataclass
    class team:
        name: str
        pool: list
        x_offset: int = 0

        def __post_init__(self):
            self.team: list = self.pool

    def damage_calculator(spell, caster, target):
        pass

    class battle_handler:
        def __init__(self, bg, y_offset, team_1, team_2):
            self.bg = bg
            self.y_offset = y_offset
            self.team_1 = team_1
            self.team_2 = team_2

            self.t1_pos = [(-32, 0, 1), (-64, -10, 1), (-80, 12, 1)]
            self.t2_pos = [(32, 0, -1), (64, -10, -1), (80, 12, -1)]
            for i,p in zip(self.team_1.pool, self.t1_pos):
                i.x, i.y, i.z = p
            for i,p in zip(self.team_2.pool, self.t2_pos):
                i.x, i.y, i.z = p

            self.turn = []

            self.caster = None
            self.spell = None
            self.target = None

            self.winner = None

            self.stat = "start"
            self.pool = []

        def hover(self, fighter):
            if not self.stat == "player_targetting":
                return
            self.target = fighter

        def start(self):
            self.pool = self.team_1.team + self.team_2.team
            random.shuffle(self.pool)
            self.stat = "wait"

        def next(self):
            self.pool.append(self.caster)
            self.caster = self.pool.pop(0)
            if self.caster in self.team_1.team:
                self.stat = "player"
            else:
                self.stat = "enemy"


        def select_spell(self, spell):
            self.spell = spell
            self.stat = "player_targetting"

        def select_target(self, target):
            if not self.stat == "player_targetting":
                return
            self.target = target
            self.stat = "casting"

        def cast(self):
            damage = 10
            self.stat = "after_cast"
        def after_cast(self):
            self.stat = "check"
            self.target = None
            self.spell = None
        def check(self):
            self.stat = "wait"

        def enemy_ai(self):
            self.stat = "wait"



screen battle_screen():
    default g = battle_handler(
        "lab_bg",
        16,
        team("Team", [Koume, Chiyako, Seiko]),
        team("Team", [Nomi, Aira, Michi]),
        )
    fixed fit_first True:
        align .5,.5 
        add g.bg
        at zoom(6,True)
        fixed:
            yoffset g.y_offset
            for i in g.team_1.pool:
                button:
                    align .5,.5 offset i.x,i.y
                    padding 0,0 background None
                    focus_mask True
                    add i.image at flip(i.z)
                    action Function(g.select_target, i)
                    hovered Function(g.hover, i)
            for i in g.team_2.pool:
                button:
                    padding 0,0 background None
                    align .5,.5 offset i.x,i.y
                    focus_mask True
                    add i.image at flip(i.z)
                    action Function(g.select_target, i)
                    hovered Function(g.hover, i)

        if g.caster:
            add "battle_caster_mark" align .5,.5 offset g.caster.x, g.caster.y-20
        if g.target:
            add "battle_target_mark" align .5,.5 offset g.target.x, g.target.y-20

    if g.stat == "start":
        timer 1 action Function(g.start)
    elif g.stat == "wait":
        timer .5 action Function(g.next)
    elif g.stat == "player":
        hbox:
            align .5,1.0 spacing 2
            at zoom(6,True)
            for i in g.caster.spells:
                button:
                    padding 0,0
                    fixed fit_first True:
                        add "fight_spell_frame"
                        add i.icon
                    action Function(g.select_spell, i)
    
    elif g.stat == "enemy":
        timer 1 action Function(g.enemy_ai)

    elif g.stat == "casting":
        timer .3 action Function(g.cast)
    elif g.stat == "after_cast":
        timer .3 action Function(g.after_cast)
    elif g.stat == "check":
        timer .3 action Function(g.check)


    text g.stat

init: # spells
    # Define all spells used in the game
    # A spell is any move or skill that can be used in fights
    # The variable name must be unique
    image force_pinch:
        Crop((0, 64, 32, 32), "rpg_battle_2/sprites.png")
    default force_pinch = spell(
        "Force pull",
        "Pinches where it lands.",
        "force_pinch", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )
    image force_impact:
        Crop((0, 96, 32, 32), "rpg_battle_2/sprites.png")
    default force_impact = spell(
        "Force impact",
        "Pushes on a single point of impact.",
        "force_impact", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )
    image force_gravity:
        Crop((0, 128, 32, 32), "rpg_battle_2/sprites.png")
    default force_gravity = spell(
        "Force impact",
        "A bigger version of force pinch.",
        "force_gravity", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )

    image poison_scratch:
        Crop((48, 64, 32, 32), "rpg_battle_2/sprites.png")
    default poison_scratch = spell(
        "Poison scratch",
        "Scratch with poisonous nails.",
        "poison_scratch", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )
    image poison_swirl:
        Crop((48, 96, 32, 32), "rpg_battle_2/sprites.png")
    default poison_swirl = spell(
        "Poison swirl",
        "A swirl of poisonous gases.",
        "poison_swirl", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )
    image poison_slash:
        Crop((48, 128, 32, 32), "rpg_battle_2/sprites.png")
    default poison_slash = spell(
        "Poison slash",
        "A slash of poisonous dust.",
        "poison_slash", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )

    image inner_fire:
        Crop((96, 64, 32, 32), "rpg_battle_2/sprites.png")
    default inner_fire = spell(
        "Inner fire",
        "Doubles your damage until you get hit.",
        "inner_fire", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )
    image fire_ball:
        Crop((96, 96, 32, 32), "rpg_battle_2/sprites.png")
    default fire_ball = spell(
        "Fireball",
        "A run-of-the-mill fireball.",
        "fire_ball", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )
    # image force_gravity:
    #     Crop((96, 128, 32, 32), "rpg_battle_2/sprites.png")
    # default poison_slash = spell(
    #     "Poison slash",
    #     "A slash of poisonous dust.",
    #     "poison_slash", # An icon can be shown instead or alongside the name.
    #     40, 0, 0, # Health damage, Mana damage, Stamina damage
    #     0, 0, 15, # Health cost, Mana cost, Stamina cost
    #     "bleed",
    #     sound = "battle/sounds/Kick.ogg"
    #     )

    image iron_punch:
        Crop((144, 64, 32, 32), "rpg_battle_2/sprites.png")
    default iron_punch = spell(
        "Iron punch",
        "The only move that Seiko knows.",
        "iron_punch", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )
    # image fire_ball:
    #     Crop((144, 96, 32, 32), "rpg_battle_2/sprites.png")
    # default fire_ball = spell(
    #     "Fireball",
    #     "A run-of-the-mill fireball.",
    #     "fire_ball", # An icon can be shown instead or alongside the name.
    #     40, 0, 0, # Health damage, Mana damage, Stamina damage
    #     0, 0, 15, # Health cost, Mana cost, Stamina cost
    #     "bleed",
    #     sound = "battle/sounds/Kick.ogg"
    #     )
    # image force_gravity:
    #     Crop((144, 128, 32, 32), "rpg_battle_2/sprites.png")
    # default poison_slash = spell(
    #     "Poison slash",
    #     "A slash of poisonous dust.",
    #     "poison_slash", # An icon can be shown instead or alongside the name.
    #     40, 0, 0, # Health damage, Mana damage, Stamina damage
    #     0, 0, 15, # Health cost, Mana cost, Stamina cost
    #     "bleed",
    #     sound = "battle/sounds/Kick.ogg"
    #     )


    image golden_cut:
        Crop((192, 64, 32, 32), "rpg_battle_2/sprites.png")
    default golden_cut = spell(
        "Golden cut",
        "You can lose your hair, or head.",
        "golden_cut", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )
    # image fire_ball:
    #     Crop((192, 96, 32, 32), "rpg_battle_2/sprites.png")
    # default fire_ball = spell(
    #     "Fireball",
    #     "A run-of-the-mill fireball.",
    #     "fire_ball", # An icon can be shown instead or alongside the name.
    #     40, 0, 0, # Health damage, Mana damage, Stamina damage
    #     0, 0, 15, # Health cost, Mana cost, Stamina cost
    #     "bleed",
    #     sound = "battle/sounds/Kick.ogg"
    #     )
    # image force_gravity:
    #     Crop((192, 128, 32, 32), "rpg_battle_2/sprites.png")
    # default poison_slash = spell(
    #     "Poison slash",
    #     "A slash of poisonous dust.",
    #     "poison_slash", # An icon can be shown instead or alongside the name.
    #     40, 0, 0, # Health damage, Mana damage, Stamina damage
    #     0, 0, 15, # Health cost, Mana cost, Stamina cost
    #     "bleed",
    #     sound = "battle/sounds/Kick.ogg"
    #     )

    image hail_packt:
        Crop((240, 64, 32, 32), "rpg_battle_2/sprites.png")
    default hail_packt = spell(
        "Hail pact",
        "A normal-sized hail with unusually high speed.",
        "hail_packt", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )
    image ice_shards:
        Crop((240, 96, 32, 32), "rpg_battle_2/sprites.png")
    default ice_shards = spell(
        "Ice shards",
        "Water that freezes after throwing.",
        "ice_shards", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )
    image frost_creep:
        Crop((240, 128, 32, 32), "rpg_battle_2/sprites.png")
    default frost_creep = spell(
        "Poison slash",
        "Frost that creeps up from beneath you.",
        "frost_creep", # An icon can be shown instead or alongside the name.
        40, 0, 0, # Health damage, Mana damage, Stamina damage
        0, 0, 15, # Health cost, Mana cost, Stamina cost
        "bleed",
        sound = "battle/sounds/Kick.ogg"
        )



init: # spell effects
    image eff bite:
        "battle/bite/1.png"
        pause .2
        alpha 0
        pause .2
        alpha 1
        pause .2
        alpha 0

    image eff bleed:
        "battle/bleed/1.png"
        pause .2
        alpha 0
        pause .2
        alpha 1
        pause .2
        alpha 0

    image eff claw:
        "battle/claw/1.png"
        pause .2
        alpha 0
        pause .2
        alpha 1
        pause .2
        alpha 0
        pause .2

    image eff fear:
        "battle/fear/1.png"
        pause .2
        alpha 0
        pause .2
        alpha 1
        pause .2
        alpha 0

    image eff peirce:
        "battle/peirce/1.png"
        pause .2
        alpha 0
        pause .2
        alpha 1
        pause .2
        alpha 0

    image eff slash:
        "battle/slash/1.png"
        pause .2
        alpha 0
        pause .2
        alpha 1
        pause .2
        alpha 0

init: # fighters
    image Koume:
        Crop((0, 0, 48, 64), "rpg_battle_2/sprites.png")
    image Chiyako:
        Crop((48, 0, 48, 64), "rpg_battle_2/sprites.png")
    image Seiko:
        Crop((96, 0, 48, 64), "rpg_battle_2/sprites.png")
    image Nomi:
        Crop((144, 0, 48, 64), "rpg_battle_2/sprites.png")
    image Aira:
        Crop((192, 0, 48, 64), "rpg_battle_2/sprites.png")
    image Michi:
        Crop((240, 0, 48, 64), "rpg_battle_2/sprites.png")

    image lab_bg = "rpg_battle_2/bg.png"

    image battle_caster_mark:
        Crop((480, 0, 16, 32), "rpg_battle_2/sprites.png")
    image battle_target_mark:
        Crop((496, 0, 16, 32), "rpg_battle_2/sprites.png")
    image fight_spell_frame:
        Crop((480, 32, 32, 32), "rpg_battle_2/sprites.png")

    default Koume = fighter(
        "Koume",
        "N/A",
        "Koume",
        100,40,40,
        [force_pinch, force_impact, force_gravity],
        )
    default Chiyako = fighter(
        "Chiyako",
        "N/A",
        "Chiyako",
        100,40,40,
        [poison_scratch, poison_swirl, poison_slash],
        )
    default Seiko = fighter(
        "Seiko",
        "N/A",
        "Seiko",
        100,40,40,
        [inner_fire, fire_ball],
        )
    default Nomi = fighter(
        "Nomi",
        "N/A",
        "Nomi",
        100,40,40,
        [iron_punch],
        )
    default Aira = fighter(
        "Aira",
        "N/A",
        "Aira",
        100,40,40,
        [golden_cut],
        )
    default Michi = fighter(
        "Michi",
        "N/A",
        "Michi",
        100,40,40,
        [hail_packt, ice_shards, frost_creep],
        )

label battle_example:
    call screen battle_screen

